pub struct Error;Expand description
Error to signal that no printable layout could be found for a document.
This means that every choice path the printer explored resulted in a
fail. Some constraints in the document will need to be relaxed for it to
be printable.