PrintResult

Struct PrintResult 

pub struct PrintResult<C: Cost> { /* private fields */ }
The resolved optimal layout for a successful print attempt.

A PrintResult is returned from Doc::validate and Doc::validate_with_cost when the printer is able to successfully produce a layout for a document.

PrintResult implements Display, so the chosen layout can be rendered through any means that allows.

Implementations

Source

impl<C: Cost> PrintResult<C>

pub fn is_tainted(&self) -> bool

Indicates if the layout chosen was tainted.

A tainted layout is one that exceeds the computation width limit imposed by the cost factory. Such a layout won’t be chosen unless there are no valid untainted layouts available. If a tainted layout is chosen, it is not guaranteed to be optimal according to the cost factory.

pub fn cost(&self) -> C

The cost of the chosen layout.