pub struct DefaultCost(pub i32, pub i32);Expand description
The default cost type used for documents.
It is composed of two values: the badness and the number of newlines. The badness quantifies how much the layout overflows the desired page width. When this cost is used, the badness will be minimized first, then the number of newlines will be minimized from there.
This cost is generally used with the DefaultCostFactory, but it is valid
to use it with a custom cost factory if it serves your needs.
Tuple Fields
0: i32The badness of the layout.
1: i32The number of newlines the layout produced.